#!/bin/bash
# ./mux.sh baudRate tmpDirectory logfile deviceName <> /dev/ttyAMA0 >&0
#
if [ $# -ne 4 ]; then
	echo "usage: mux.sh baudRate tmpDirectory logfile deviceName"
	exit 0
fi
# debugging file.  Set to /dev/null to disable logging
if [ "$3" == "/dev/null" ]; then
	logfile=$3
else
	logfile=$2/$3
fi
#
# temporary file to hold the HTTP POST to the cloud
tmpfile=$2/dweet$$
#
# temporary file to hold the cloud response
rdfile=$2/dweetRsp$$
#
# device name for the cloud post
deviceName=$4
#
# a few global script variables
c=""
readIn=""
postOut=""
postRunning=0
lcnt=0
#
# destroy the mux instances before exit
#
cleanExit()
{
	echo " "
	read -t 1 junk
	echo "1k"
	read -t 1 junk
	echo "2k"
	read -t 1 junk
	stty 9600 sane
	exit 0
}
#
# Build the cloud GET message header (for posting data to the cloud) & send it
#
doDweet()
{
	echo -n "GET /dweet/for/$deviceName?" > $tmpfile
	echo -n "$1" >> $tmpfile
	echo -n " HTTP/1.1" >> $tmpfile
	echo -e "\x0D" >> $tmpfile
	echo -n "Host: dweet.io" >> $tmpfile
	echo -e "\x0D" >> $tmpfile
	echo -n "User-Agent: xPicoPiPlate/1.0" >> $tmpfile
	echo -e "\x0D" >> $tmpfile
	echo -n "Accept: text/html,application/xhtml+xml" >> $tmpfile
	echo -n ",application/xml" >> $tmpfile
	echo -n ";q=0.9,*/*;q-0.8" >> $tmpfile
	echo -e "\x0D" >> $tmpfile
	echo -n "Accept-Language: en-US,en;q=0.5" >> $tmpfile
	echo -e "\x0D" >> $tmpfile
	echo -n "Accept-Encoding: identity" >> $tmpfile
	echo -e "\x0D" >> $tmpfile
	echo -n "Connection: keep-alive" >> $tmpfile
	echo -e "\x0D" >> $tmpfile
	echo -n "Cache-Control: max-age=0" >> $tmpfile
	echo -e "\x0D" >> $tmpfile
	echo -e "\x0D" >> $tmpfile
		# connect to the cloud service
	if [ $postRunning -eq 0 ]; then
		lcnt=0
		postRunning=1
		doCmd "2cdweet.io:80"
		if [ "$cmdRsp" != "K" ]; then
				# failure, kill instance
			postRunning=0
			doCmd 2k
		fi
	else
		lcnt=`expr $lcnt + 1`
		if [ $lcnt -ge 3 ]; then	# missed
			postRunning=0
			doCmd 2k
		fi
	fi
}
#
# read response data from cloud service (upto 1024 bytes)
# and stuff it in a file ($2)
#
muxReadBinaryFile()
{
	cat /dev/null  > $2
	doCmd "$1rb*1024"
	if  [ "$cmdRsp" == "K" ]; then
		while [ 1 ]
		do
			c=
			read -t 1 -n 1 c
			if [ $? -eq 0 ]; then
				if [ "$c" == "*" ]; then
					c1=
					read -t 1 -n 1 c1
					if [ "$c1" != "*" ]; then
						break
					fi
				fi
				echo -n "$c" >> $2
			else
				break
			fi
		done
		endRead $1
	fi
}
#
# read POSTed data from web browser (upto 256 bytes)
# and stuff it in a variable readIn
#
rdEvent()
{
	doCmd "$1rb*256"
	if  [ "$cmdRsp" == "K" ]; then
#		 	read POST message (luckily, it terminates with newline)
		read -t 5 readIn
		endRead $1
	fi
}
#
# terminate the mux read operation
endRead()
{
#		send command byte to end mux operation
	echo -n "$1"
#               read should be "*\n"
	read -t 1 junk
#		send command terminator
	echo ""
#               read should be "K" or "W"
	c=
	read -t 1 -n 1 c
}
#
# write binary data to the mux
muxWriteBinary()
{
	echo "$1sb~"
	incnt=
	c=
#		Get the mux buffer size
	while [[ "$c" != "K" ]]
	do
		read -n 1 c
		case "$c" in
			E) 	# error message
				read -t 1 junk
				break
				;;
			K)
				break
				;;
			*)
				incnt=$incnt$c
				;;
		esac
	done
	if [ "$c" == "K" ]; then
#			calculate buffer size needed
		if [ "$1" == "1" ]; then
			dataBytes=${#2}
		elif [ "$1" == "2" ]; then
			dataBytes=$(wc -c "$2" | cut -f 1 -d ' ')
		fi
		if [ $incnt -gt $dataBytes ]; then
			if [ "$1" == "1" ]; then
				echo -n $2
			elif [ "$1" == "2" ]; then
				cat $2
			fi
			echo "~"
			read -t 5 -n 1 c
		fi
	fi
}
#
# Open a new listen socket
# $1 - instance
# $2 - close & open options
muxListen()
{
	case "$2" in
		0)	doCmd $1h
			doCmd W$1r
			;;
		1)	doCmd $1k
			doCmd $1h
			doCmd W$1r
			;;
		2)	doCmd $1p
			doCmd $1f
			doCmd $1e
			doCmd $1h
			;;
	esac
}
#
# wait for a mux event
waitEvents()
{
	kcnt=0
	while [ 1 ]
	do
		eList=""
		evt=""
		read -t 5 eList
		if [ $? -ne 0 ]; then
			if [ $postRunning -eq 1 ]; then
				doCmd 2k
				postRunning=0
				doCmd W1r
			else 
#					check channel
				doCmd 1
				if  [ "$cmdRsp" == "D" ]; then
					muxListen 1 0
					kcnt=0
				elif  [ "$cmdRsp" == "E" ]; then
					muxListen 1 1
					kcnt=0
				elif  [ "$cmdRsp" == "K" ]; then
					kcnt=`expr $kcnt + 1`
					if [ $kcnt -ge 3 ]; then
						muxListen 1 1
						kcnt=0
					fi
				fi
			fi
		else
			evt=`echo $eList | cut -b1-2`
		fi
		case "$evt" in
			1r) 	# read ready on channel 1
				kcnt=0
				rdEvent 1
				resp1=`./hvacSim.sh -r "$readIn" $logfile`
				muxWriteBinary 1 "$resp1"
				muxListen 1 2
				if [ ${#deviceName} -ne 0 ]; then
					postOut=`./hvacSim.sh -p "$readIn" $logfile`
					doDweet $postOut
				fi
				if [ $postRunning -eq 1 ]; then
					doCmd W1r2s
				else
					doCmd W1r
				fi
				;;
			2r) 	# read ready on channel 2
				muxReadBinaryFile 2 "$rdfile"
				doCmd 2e
				postRunning=0
				doCmd W1r
				;;
			2s) 	#send ready on channel 2
				muxWriteBinary 2 "$tmpfile"
				doCmd 2p
				doCmd W2r
				;;
		esac
	done
}
#
# simple send command and parse response
# $1 - command
# if not in sync, this read operation will block - that's bad
doCmd()
{
	echo "$1"
	junk=
	cmdRsp=
	while [ "$cmdRsp" == "" ]
	do
	  read -n 1 cmdRsp
	  if  [ "$cmdRsp" != "" ]; then
	    case "$cmdRsp" in
		E)	read -t 1 junk 
			;;
		K)	;;
		D)	;;
		W)	;;
		*)	
			read -t 1 junk
			cmdRsp="U"
			;;
	    esac
	  fi
	done
}
#
# sync up with mux inteperter
syncToStart()
{
# we don't know if stale data is available
#			read until there is nothing
	echo "1"
	while [ 1 ]
	do
		read -t 1 -n 1 junk
		if [ $? -ne 0 ]; then
			break
		fi
	done
#			send status command to sync with mux parser
	doCmd 1
#			kill both instances - just in case we are mid operation
	doCmd 1k
	doCmd 2k
}
#
# Main script entry
# 
# open the log, and disable tty echo and output processing
#
trap cleanExit 1 2 3
echo "open log $$" > $logfile
echo "HTTP mux" >> $logfile
stty $1 -echo -opost -icanon
			# clear & sync the communication channel
syncToStart
			# wait for events - never returns
waitEvents
			# clean up and exit
cleanExit
#
